#include "Play.hpp"
#include "replicate.hpp"

#include <Graphics/Graphics.hpp>
#include <Graphics/Texture.hpp>
#include <Graphics/Font.hpp>
#include <Vector/VectorGL.hpp>

#define GRAV_CONST .00005
#define MIN_SQR_DIST 0.0001
#define FRICTION 0.9
#define STARTING_BALLS 10
#define INIT_SPEED_CONST 0.01
#define MASS_TO_AREA_RATIO 0.005


float rand_float()
{
  return (float) random()/RAND_MAX;
}

Play::Play() {
  uint i;
  Vector2f spos;
  Vector2f sspd;
  //make the board
  std::vector<Vector2f> corners(4);
  corners[0] = make_vector(-1.0f,-1.0f);
  corners[1] = make_vector(1.0f,-1.0f);
  corners[2] = make_vector(1.0f,1.0f);
  corners[3] = make_vector(-1.0f,1.0f);
  std::vector<Ball> balls(STARTING_BALLS);
  //make the balls
  for(i=0; i<balls.size(); i++)
  {
    spos = 2.0f * make_vector(rand_float(), rand_float()) - make_vector(1.0f,1.0f);
    sspd = INIT_SPEED_CONST * make_vector(rand_float(), rand_float());
    balls[i] = Ball::Ball(spos, sspd, 0.01f);
  }
  state = Replicate::Replicate(corners, balls);
  state.total_mass = 10;
  state.time = 0;
  state.dwl = 0;
}

Play::~Play() {
}

void Play::draw() {
	if(state.dwl !=0)
  {
    if(state.dwl == 1)
    {
/*      glColor3f(1.0f,0.0f,0.0f);
      glBegin(GL_POLYGON);
      glVertex(make_vector(1.0f, 1.0f));
      glVertex(make_vector(1.0f, -1.0f));
      glVertex(make_vector(-1.0f, -1.0f));
      glVertex(make_vector(-1.0f, 1.0f));
      glEnd();
      */
    }
    else
    {
      glColor3f(1.0f,0.0f,0.0f);
      glBegin(GL_POLYGON);
      glVertex(make_vector(1.0f, 1.0f));
      glVertex(make_vector(1.0f, -1.0f));
      glVertex(make_vector(-1.0f, -1.0f));
      glVertex(make_vector(-1.0f, 1.0f));
      glEnd();
    }
    return;
  }
  
  glClearColor(0.5f, 0.5f, 0.5f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
  //setup for drawing
  uint i;
 
 
  //Draw the board
  glColor3f(1.0f,1.0f,1.0f);
  glBegin(GL_POLYGON);
  for(i=0; i < state.corners.size(); i++)
  {
    glVertex2f(state.corners[i].x, state.corners[i].y); 
  }
  glEnd();
 
  //draw the status bar
  //time
  float time_dist = (2.0f * state.time/state.final_time) - 1.0f;
  glColor3f(0.0f, 0.0f, 0.0f);
  glLineWidth(10.0f);
  glBegin(GL_LINES);
  glVertex(make_vector(-1.0f, .95f));
  glVertex(make_vector(time_dist, .95f));
  glEnd();
  //total mass
  float mass_dist = (2.0f * state.total_mass/state.final_mass) - 1.0f;
  glColor3f(0.0f, 0.0f, 0.0f);
  glLineWidth(10.0f);
  glBegin(GL_LINES);
  glVertex(make_vector(.95f, -1.0f));
  glVertex(make_vector(.95f, mass_dist));
  glEnd();
 
  //Draw the balls,,,,,, now
  float posx, posy, radius;
  for(i=0; i < state.balls.size(); i++)
  {
    posx = state.balls[i].pos.x;
    posy = state.balls[i].pos.y;
    radius = sqrt(state.balls[i].mass)*MASS_TO_AREA_RATIO;
    glColor3f(state.balls[i].color.x, state.balls[i].color.y, state.balls[i].color.z); 
    glBegin(GL_POLYGON);
    glVertex2f(posx + radius, posy + radius);
    glVertex2f(posx - radius, posy + radius);
    glVertex2f(posx - radius, posy - radius);
    glVertex2f(posx + radius, posy - radius);
    glEnd();
  }

  //draw the mouse
  glColor3f(0.0f,0.0f,0.0f); 
  glLineWidth(3.0f);
  glBegin(GL_LINES);
  glVertex(mouse_pos + make_vector(0.0f, 0.05f));
  glVertex(mouse_pos + make_vector(0.0f, -0.05f));
  glEnd();
  glBegin(GL_LINES);
  glVertex(mouse_pos + make_vector(0.05f, 0.0f));
  glVertex(mouse_pos + make_vector(-0.05f, 0.0f));
  glEnd();

}

void Play::update(float elapsed_time) {
  uint i;
  Ball cur_ball;
  float sqr_dist;
  //handle game winning and losing

  if(state.dwl!=0)
  {
    return;
  }

  if(state.time < state.final_time && state.total_mass > state.final_mass)
  {
    state.dwl = 1;
  }
  if(state.time > state.final_time && state.total_mass < state.final_mass)
  {
    state.dwl = 2;
  }
  state.time+=1;
  
  //update the state of all balls

  for(i=0; i<state.balls.size(); i++)
  {
    //set cur_ball to the current ball in the loop
    cur_ball = state.balls[i];
    //check if the ball is within bouncing distance of a wall
    //first for x reversal
    if(cur_ball.pos.x + cur_ball.rad > 1.0f || cur_ball.pos.x - cur_ball.rad < -1.0f)
    {
      cur_ball.spd.x = -cur_ball.spd.x;
    }
    //then for y reversal
    if(cur_ball.pos.y + cur_ball.rad > 1.0f || cur_ball.pos.y - cur_ball.rad < -1.0f)
    {
      cur_ball.spd.y = -cur_ball.spd.y;
    }
    //update the ball speed based on the mouse pos, try to do some relationship inversely proportional to the distance away
    sqr_dist = pow(cur_ball.pos.x - mouse_pos.x,2) + pow(cur_ball.pos.y - mouse_pos.y,2);
    if(sqr_dist > MIN_SQR_DIST) cur_ball.spd += GRAV_CONST * 1/sqr_dist * (mouse_pos - cur_ball.pos);
    //balls are always subject to friction
    cur_ball.spd = cur_ball.spd * FRICTION;

    //add the speed to each ball
    cur_ball.pos += cur_ball.spd;
    //put the info back into the state
    state.balls[i] = cur_ball;
  }

  //gather the balls
  state.gather_balls();

  //blow up balls if applicable
  if(!active)
  {
    int mouse_ind_x = (int)((mouse_pos.x + 1.0f)*state.grid_size_x);
    int mouse_ind_y = (int)((mouse_pos.y + 1.0f)*state.grid_size_y);
    for(i=0; i<state.balls.size(); i++)
    {
      if(mouse_ind_x == state.balls[i].x_ind && mouse_ind_y == state.balls[i].y_ind)
      {
        state.blowup_ball(i);
      }
    }
  }
}

void Play::handle_event(SDL_Event const &event) {
	if (event.type == SDL_MOUSEMOTION) {
		mouse_pos.x = event.motion.x / float(Graphics::screen.x) * 2.0f * Graphics::aspectf() - Graphics::aspectf();
		mouse_pos.y = event.motion.y / float(Graphics::screen.y) *-2.0f + 1.0f;
	}
	if (event.type == SDL_MOUSEBUTTONDOWN) {
		active = false;
	}
	if (event.type == SDL_MOUSEBUTTONUP) {
		active = true;
	}

  
  if (event.type == SDL_KEYDOWN) {
		if (event.key.keysym.sym == SDLK_ESCAPE) {
			//escape quits.
			quit_screen(this);
		}
	}
}


